/**
 * vim: set ts=4 :
 * =============================================================================
 * Builtin Votes BaseVote Class
 * Copyright (C) 2011 Ross Bemrose (Powerlord).  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "Votes.h"

// BaseVote
bool BaseVote::AppendItem(const char *info)
{
	if (m_pVoteOptions.size() >= MAX_VOTE_OPTIONS)
	{
		return false;
	}
	m_pVoteOptions.push_back(info);
	return true;
}

bool BaseVote::InsertItem(unsigned int position, const char *info)
{
	if (m_pVoteOptions.size() >= MAX_VOTE_OPTIONS)
	{
		return false;
	}

	if (position >= m_pVoteOptions.size())
	{
		return false;
	}
	CVector<const char *>::iterator iter = m_pVoteOptions.iterAt(position);
	m_pVoteOptions.insert(iter, info);
	
	return true;
}

bool BaseVote::RemoveItem(unsigned int position)
{
	if (position >= m_pVoteOptions.size())
	{
		return false;
	}

	m_pVoteOptions.erase(m_pVoteOptions.iterAt(position));

	return true;
}

void BaseVote::RemoveAllItems()
{
	m_pVoteOptions.clear();
}

const char *BaseVote::GetItemInfo(unsigned int position)
{
	if (position >= m_pVoteOptions.size())
	{
		return NULL;
	}

	return m_pVoteOptions.at(position);
}

unsigned int BaseVote::GetItemCount()
{
	return m_pVoteOptions.size();
}

BuiltinVoteType BaseVote::GetVoteType()
{
	return m_voteType;
}

const char *BaseVote::GetVoteArgument()
{
	return m_voteArgument;
}

void BaseVote::SetVoteArgument(const char *voteArgument)
{
	m_voteArgument = voteArgument;
}

byte BaseVote::GetTeam()
{
	return m_Team;
}

void BaseVote::SetTeam(byte team)
{
	m_Team = team;
}

unsigned int BaseVote::GetAllPlayers(cell_t clients[])
{
	unsigned int playersNum = 0;

	for (int i = 1; i <= playerhelpers->GetMaxClients(); i++)
	{
		if (playerhelpers->GetGamePlayer(i)->IsInGame())
		{
			clients[playersNum++] = i;
		}
	}
	return playersNum;
}

void BaseVote::InternalDelete()
{
	if (m_bWillFreeHandle && m_hHandle != BAD_HANDLE)
	{
		Handle_t hndl = m_hHandle;
		HandleSecurity sec;

		sec.pOwner = m_pOwner;
		sec.pIdentity = NULL;

		m_hHandle = BAD_HANDLE;
		m_bDeleting = true;
		handlesys->FreeHandle(hndl, &sec);
	}

}

void BaseVote::Destroy(bool releaseHandle)
{
	if (m_bDeleting)
	{
		return;
	}

	// m_bWillFreeHandle = releaseHandle;

	if (!m_bCancelling || m_bShouldDelete)
	{
		Cancel();
		InternalDelete();
	} else {
		m_bShouldDelete = true;
	}
}

void BaseVote::Cancel()
{
// Not sure what should go here
	DisplayVoteFail(BuiltinVoteFail_Generic);
}

bool BaseVote::IsYesNoVote()
{
	return m_bYesNo;
}

int BaseVote::GetVoteInitiator()
{
	return m_voteInitiator;
}

Handle_t BaseVote::GetHandle()
{
	return m_hHandle;
}

void BaseVote::SetResultHandler(IPluginFunction *pVoteResults)
{
	m_pVoteResults = pVoteResults;
}

void BaseVote::DisplayAllVotes(unsigned int totalClients, unsigned int yesVotes, unsigned int noVotes)
{
	// noop
}

void BaseVote::SendOptions()
{
	// noop
}

// Old handler stuff

void BaseVote::OnBuiltinVoteStart()
{
	if ((m_Flags & (int)BuiltinVoteAction_Start) == (int)BuiltinVoteAction_Start)
	{
		DoAction(BuiltinVoteAction_Start, 0, 0);
	}
}

void BaseVote::OnBuiltinVoteSelect(int client, unsigned int item)
{
	DoAction(BuiltinVoteAction_Select, client, item);
}

void BaseVote::OnBuiltinVoteCancel(BuiltinVoteFailReason reason)
{
	DoAction(BuiltinVoteAction_Cancel, (cell_t) reason, 0);
}

void BaseVote::OnBuiltinVoteEnd(BuiltinVoteEndReason reason)
{
	DoAction(BuiltinVoteAction_End, reason, 0);
}

void BaseVote::OnBuiltinVoteDestroy()
{
	// Not sure what to do here... frees VoteHandler in old code
}

void BaseVote::OnBuiltinVoteVoteStart()
{
	DoAction(BuiltinVoteAction_VoteStart, 0, 0);
}

cell_t BaseVote::DoAction(BuiltinVoteAction action, cell_t param1, cell_t param2, cell_t def_res)
{
	cell_t res = def_res;
	m_pHandler->PushCell(GetHandle());
	m_pHandler->PushCell((cell_t)action);
	m_pHandler->PushCell(param1);
	m_pHandler->PushCell(param2);
	m_pHandler->Execute(&res);
	return res;
}

void BaseVote::OnBuiltinVoteResults(const builtinvote_result_t *results)
{
	if (!m_pVoteResults)
	{
		/* Call BuiltinVoteAction_End instead.  See if there are any extra winners. */
		unsigned int num_items = 1;
		for (unsigned int i = 1; i<results->num_items; i++)
		{
			if (results->item_list[i].count != results->item_list[0].count)
			{
				break;
			}
			num_items++;
		}

		/* See if we need to pick a random winnner. */
		unsigned int winning_item;
		if (num_items > 1)
		{
			/* Yes, we do. */
			srand(time(NULL));
			winning_item = rand() % num_items;
			winning_item = results->item_list[winning_item].item;
		} else {
			/* No, take the first. */
			winning_item = results->item_list[0].item;
		}

		unsigned int total_votes = results->num_votes;
		unsigned int winning_votes = results->item_list[0].count;

		DoAction(BuiltinVoteAction_End, winning_item, (total_votes << 16) | (winning_votes & 0xFFFF));
	} else {
		IPluginContext *pContext = m_pVoteResults->GetParentContext();
		bool no_call = false;
		int err = SP_ERROR_NONE;

		/* First array */
		cell_t client_array_address = -1;
		cell_t *client_array_base = NULL;
		cell_t client_array_size = results->num_clients + (results->num_clients * 2);
		if (client_array_size)
		{
			if ((err = pContext->HeapAlloc(client_array_size, &client_array_address, &client_array_base))
				!= SP_ERROR_NONE)
			{
				//g_DbgReporter.GenerateError(pContext, m_fnVoteResult, err, "Menu callback could not allocate %d bytes for client list.", client_array_size * sizeof(cell_t));
				no_call = true;
			} else {
				cell_t target_offs = sizeof(cell_t) * results->num_clients;
				cell_t *cur_index = client_array_base;
				cell_t *cur_array;
				for (unsigned int i = 0; i < results->num_clients; i++)
				{
					/* Copy the array index */
					*cur_index = target_offs;
					/* Get the current array address */
					cur_array = (cell_t *)((char *)cur_index + target_offs);
					/* Store information */
					cur_array[0] = results->client_list[i].client;
					cur_array[1] = results->client_list[i].item;
					/* Adjust for the new target by subtracting on indirection
					 * and adding one array.
					 */
					target_offs += (sizeof(cell_t) * 2) - sizeof(cell_t);
					cur_index++;
				}
			}
		}

		/* Second array */
		cell_t item_array_address = -1;
		cell_t *item_array_base = NULL;
		cell_t item_array_size = results->num_items + (results->num_items * 2);
		if (item_array_size)
		{
			if ((err - pContext->HeapAlloc(item_array_size, &item_array_address, &item_array_base))
				!= SP_ERROR_NONE)
			{
				// g_DbgReporter.GenerateError(pContext, m_fnVoteResult, err, "Menu callback could not allocate %d bytes for item list.", item_array_size);
				no_call = true;
			} else {
				cell_t target_offs = sizeof(cell_t) * results->num_items;
				cell_t *cur_index = item_array_base;
				cell_t *cur_array;
				for (unsigned int i = 0; i < results->num_items; i++)
				{
					/* Copy the array index */
					*cur_index = target_offs;
					/* Get the current array address */
					cur_array = (cell_t *)((char *)cur_index + target_offs);
					/* Store infomation */
					cur_array[0] = results->item_list[i].item;
					cur_array[1] = results->item_list[i].count;

					/* Adjust for the new target by subtracting one indirection
					 * and adding one array.
					 */
					target_offs += (sizeof(cell_t) * 2) - sizeof(cell_t);
					cur_index++;
				}
			}
		}

		/* Finally, push everything */
		if (!no_call)
		{
			m_pVoteResults->PushCell(GetHandle());
			m_pVoteResults->PushCell(results->num_votes);
			m_pVoteResults->PushCell(results->num_clients);
			m_pVoteResults->PushCell(client_array_address);
			m_pVoteResults->PushCell(results->num_items);
			m_pVoteResults->PushCell(item_array_address);
			m_pVoteResults->Execute(NULL);
		}

		/* Free what we allocated, in reverse order as required */
		if (item_array_address != -1)
		{
			pContext->HeapPop(item_array_address);
		}

		if (client_array_address != 1)
		{
			pContext->HeapPop(client_array_address);
		}
	}
}

BaseVote::BaseVote(IPluginFunction *pHandler, IdentityToken_t *pOwner, BuiltinVoteType voteType, bool bYesNo) :
m_pHandler(pHandler), m_pOwner(pOwner), m_voteType(voteType), m_bYesNo(bYesNo)
{
	if (bYesNo)
	{
		m_pVoteOptions.push_back("Yes");
		m_pVoteOptions.push_back("No");
	}
}

TF2Vote::TF2Vote(IPluginFunction *pHandler, IdentityToken_t *pOwner, BuiltinVoteType voteType, bool bYesNo) :
BaseVote(pHandler, pOwner, voteType, bYesNo)
{
}

unsigned int TF2Vote::GetApproxMemUsage()
{
	return sizeof(TF2Vote);
}

bool TF2Vote::SetVoteType(BuiltinVoteType voteType)
{
	switch(voteType)
	{
	case BuiltinVoteType_Kick:
	case BuiltinVoteType_KickIdle:
	case BuiltinVoteType_KickScamming:
	case BuiltinVoteType_KickCheating:
	case BuiltinVoteType_Restart:
	case BuiltinVoteType_ChangeLevel:
	case BuiltinVoteType_NextLevel:
	case BuiltinVoteType_NextLevelChoices:
	case BuiltinVoteType_ScrambleImmediate:
	case BuiltinVoteType_ScrambleRoundEnd:
	case BuiltinVoteType_Custom:
		m_voteType = voteType;
		return true;
		break;
	default:
		return false;
		break;
	}
}

void TF2Vote::DisplayVoteFail(BuiltinVoteFailReason reason)
{
	// Display the vote fail message
	cell_t clients[256];
	unsigned int playersNum = GetAllPlayers(clients);

	int msgId = usermsgs->GetMessageIndex("VoteFailed");

	bf_write *bf = usermsgs->StartMessage(msgId, clients, playersNum, USERMSG_RELIABLE);
	bf->WriteByte(TEAMS_ALL);
	bf->WriteByte(reason);
	usermsgs->EndMessage();
}

void TF2Vote::DisplayVotePass(const char *winner)
{
	char *translation;

	switch (m_voteType)
	{
	
	case BuiltinVoteType_Kick:
	case BuiltinVoteType_KickIdle:
	case BuiltinVoteType_KickScamming:
	case BuiltinVoteType_KickCheating:
		translation = TRANSLATION_TF2_VOTE_KICK_PASSED;
		break;

	case BuiltinVoteType_Restart:
		translation = TRANSLATION_TF2_VOTE_RESTART_PASSED;
		break;

	case BuiltinVoteType_ChangeLevel:
		translation = TRANSLATION_TF2_VOTE_CHANGELEVEL_PASSED;
		break;

	case BuiltinVoteType_NextLevel:
	case BuiltinVoteType_NextLevelChoices:
		translation = TRANSLATION_TF2_VOTE_NEXTLEVEL_PASSED;
		break;

	case BuiltinVoteType_ScrambleImmediate:
	case BuiltinVoteType_ScrambleRoundEnd:
		translation = TRANSLATION_TF2_VOTE_SCRAMBLE_PASSED;
		break;

	default:
		translation = TRANSLATION_TF2_VOTE_CUSTOM;
		break;

	}

	// Display the vote pass message
	cell_t clients[256];
	unsigned int playersNum = GetAllPlayers(clients);

	int msgId = usermsgs->GetMessageIndex("VotePass");

	bf_write *bf = usermsgs->StartMessage(msgId, clients, playersNum, USERMSG_RELIABLE);
	bf->WriteByte(TEAMS_ALL);
	bf->WriteString(translation);
	bf->WriteString(winner);
	usermsgs->EndMessage();
}

void TF2Vote::SendOptions()
{
	char prefix[] = "option";

	IGameEvent *optionsEvent = events->CreateEvent("vote_options");
	
	for (unsigned int i = 0; i < GetItemCount(); i++)
	{
		optionsEvent->SetString(prefix + (i + 1), GetItemInfo(i));
	}

	events->FireEvent(optionsEvent);
}

void TF2Vote::ShowVote(cell_t clients[], unsigned int playersNum, const char *voteArgument)
{
	int msgId = usermsgs->GetMessageIndex("VoteStart");
	char *translation;

	switch (m_voteType)
	{
	
	case BuiltinVoteType_Kick:
		translation = TRANSLATION_TF2_VOTE_KICK_START;
		break;

	case BuiltinVoteType_KickIdle:
		translation = TRANSLATION_TF2_VOTE_KICK_IDLE_START;
		break;

	case BuiltinVoteType_KickScamming:
		translation = TRANSLATION_TF2_VOTE_KICK_SCAMMING_START;
		break;

	case BuiltinVoteType_KickCheating:
		translation = TRANSLATION_TF2_VOTE_KICK_CHEATING_START;
		break;

	case BuiltinVoteType_Restart:
		translation = TRANSLATION_TF2_VOTE_RESTART_START;
		break;

	case BuiltinVoteType_ChangeLevel:
		translation = TRANSLATION_TF2_VOTE_CHANGELEVEL_START;
		break;

	case BuiltinVoteType_NextLevel:
		translation = TRANSLATION_TF2_VOTE_NEXTLEVEL_SINGLE_START;
		break;

	case BuiltinVoteType_NextLevelChoices:
		translation = TRANSLATION_TF2_VOTE_NEXTLEVEL_MULTIPLE_START;
		break;

	case BuiltinVoteType_ScrambleImmediate:
		translation = TRANSLATION_TF2_VOTE_SCRAMBLE_IMMEDIATE_START;
		break;

	case BuiltinVoteType_ScrambleRoundEnd:
		translation = TRANSLATION_TF2_VOTE_SCRAMBLE_ROUNDEND_START;
		break;

	default:
		translation = TRANSLATION_TF2_VOTE_CUSTOM;
		break;

	}

	bf_write *bf = usermsgs->StartMessage(msgId, clients, playersNum, USERMSG_RELIABLE);
	bf->WriteByte(TEAMS_ALL);
	bf->WriteByte(m_voteInitiator);
	bf->WriteString(translation);
	bf->WriteString(voteArgument);
	bf->WriteOneBit(m_bYesNo);
	usermsgs->EndMessage();
}

void TF2Vote::ShowVoteCast(int entityId, int item)
{
	//Fire the vote_cast event
	IGameEvent *castEvent = events->CreateEvent("vote_cast");
	castEvent->SetInt("team", -1);
	castEvent->SetInt("entityid", entityId);
	castEvent->SetInt("vote_option", item);
	events->FireEvent(castEvent);
}

void TF2Vote::CastVote(edict_t *pEntity, const CCommand &args)
{
	if (!pEntity || pEntity->IsFree())
	{
		return;
	}

	const char *cmd = args.Arg(0);
	if (strcmp(cmd, "vote") == 0)
	{
		// All of format "option?" where ? is 1 through 5
		const char voteCast = args[1][6];

		// except that Valve uses 0-4 internally, and so do we (in arrays)
		int item = (int) strtol(&voteCast, NULL, 10) - 1;
		IGamePlayer *player = playerhelpers->GetGamePlayer(pEntity);
		int entityId = player->GetUserId();
		int client = playerhelpers->GetClientOfUserId(entityId);

		ShowVoteCast(entityId, item);
		OnBuiltinVoteSelect(client, item);

		RETURN_META(MRES_SUPERCEDE);
	}

	RETURN_META(MRES_IGNORED);
}

/*
void TF2Vote::DisplayAllVotes()
{
	// noop

}
*/

L4DVote::L4DVote(IPluginFunction *pHandler, IdentityToken_t *pOwner, BuiltinVoteType voteType) :
BaseVote(pHandler, pOwner, voteType, true)
{
}

unsigned int L4DVote::GetApproxMemUsage()
{
	return sizeof(L4DVote);
}

void L4DVote::VoteEnded()
{
	IGameEvent *endEvent = events->CreateEvent("vote_ended");
	events->FireEvent(endEvent);
}

void L4DVote::DisplayVoteFail(BuiltinVoteFailReason reason)
{
	VoteEnded();
	// reason is ignored in L4D
	IGameEvent *failEvent = events->CreateEvent("vote_failed");
	failEvent->SetInt("team", TEAMS_ALL);
	events->FireEvent(failEvent);
}

void L4DVote::DisplayVotePass(const char *winner)
{
	char *translation;
	switch(m_voteType)
	{
	case BuiltinVoteType_Kick:
		translation = TRANSLATION_L4D_VOTE_KICK_PASSED;
		break;

	case BuiltinVoteType_Restart:
		translation = TRANSLATION_L4D_VOTE_RESTART_PASSED;
		break;

	case BuiltinVoteType_ChangeLevel:
		translation = TRANSLATION_L4D_VOTE_CHANGELEVEL_PASSED;
		break;

	case BuiltinVoteType_ChangeCampaign:
		translation = TRANSLATION_L4D_VOTE_CHANGECAMPAIGN_PASSED;
		break;

	case BuiltinVoteType_ReturnToLobby:
		translation = TRANSLATION_L4D_VOTE_RETURNTOLOBBY_PASSED;
		break;

	case BuiltinVoteType_ChangeDifficulty:
		translation = TRANSLATION_L4D_VOTE_CHANGEDIFFICULTY_PASSED;
		break;

	case BuiltinVoteType_Custom:
		translation = TRANSLATION_L4D_VOTE_CUSTOM;
		break;

	}

	VoteEnded();
	IGameEvent *passEvent = events->CreateEvent("vote_passed");
	passEvent->SetString("details", translation);
	passEvent->SetString("param1", winner);
	passEvent->SetInt("team", TEAMS_ALL);
	events->FireEvent(passEvent);
}

/*
bool L4DVote::SendOptions()
{
	// noop
	return false;
}
*/

bool L4DVote::SetVoteType(BuiltinVoteType voteType)
{
	switch(voteType)
	{
	case BuiltinVoteType_Kick:
	case BuiltinVoteType_Restart:
	case BuiltinVoteType_ChangeLevel:
	case BuiltinVoteType_ChangeCampaign:
	case BuiltinVoteType_ReturnToLobby:
	case BuiltinVoteType_ChangeDifficulty:
	case BuiltinVoteType_Custom:
		m_voteType = voteType;
		return true;
		break;

	default:
		return false;
		break;
		
	}
}

void L4DVote::ShowVote(cell_t clients[], unsigned int playersNum, const char *voteArgument)
{
	char *translation;
	switch(m_voteType)
	{
	case BuiltinVoteType_Kick:
		translation = TRANSLATION_L4D_VOTE_KICK;
		break;

	case BuiltinVoteType_Restart:
		translation = TRANSLATION_L4D_VOTE_RESTART_START;
		break;

	case BuiltinVoteType_ChangeLevel:
		translation = TRANSLATION_L4D_VOTE_CHANGELEVEL_START;
		break;

	case BuiltinVoteType_ChangeCampaign:
		translation = TRANSLATION_L4D_VOTE_CHANGECAMPAIGN_START;
		break;

	case BuiltinVoteType_ReturnToLobby:
		translation = TRANSLATION_L4D_VOTE_RETURNTOLOBBY_START;
		break;

	case BuiltinVoteType_ChangeDifficulty:
		translation = TRANSLATION_L4D_VOTE_CHANGEDIFFICULTY_START;
		break;

	case BuiltinVoteType_Custom:
		translation = TRANSLATION_L4D_VOTE_CUSTOM;
		break;

	}

	IGameEvent *startEvent = events->CreateEvent("vote_started");
	startEvent->SetInt("team", TEAMS_ALL);
	startEvent->SetInt("intiator", m_voteInitiator);
	startEvent->SetString("issue", translation);
	startEvent->SetString("param1", voteArgument);
}

void L4DVote::ShowVoteCast(int entityId, int item)
{
	IGameEvent *castEvent;

	switch (item)
	{
		case VOTE_YES_INDEX :
			castEvent = events->CreateEvent("vote_cast_yes");
			break;

		case VOTE_NO_INDEX :
			castEvent = events->CreateEvent("vote_cast_no");
			break;

	}

	castEvent->SetInt("team", TEAMS_ALL);
	castEvent->SetInt("entityid", entityId);
	events->FireEvent(castEvent);
}

void L4DVote::CastVote(edict_t *pEntity, const CCommand &args)
{
	if (!pEntity || pEntity->IsFree())
	{
		return;
	}

	const char *cmd = args.Arg(0);
	if (stricmp(cmd, "Vote") == 0)
	{

		// The text "Yes" or "No"
		int item;

		if (stricmp(args.Arg(1), "Yes"))
		{
			item = VOTE_YES_INDEX;
		}
		else
		{
			item = VOTE_NO_INDEX;
		}

		IGamePlayer *player = playerhelpers->GetGamePlayer(pEntity);
		
		int client = playerhelpers->GetClientOfUserId(player->GetUserId());

		ShowVoteCast(client, item);
		OnBuiltinVoteSelect(client, item);

		RETURN_META(MRES_SUPERCEDE);
	}

	RETURN_META(MRES_IGNORED);
}

void L4DVote::DisplayAllVotes(unsigned int totalClients, unsigned int yesVotes, unsigned int noVotes)
{
	IGameEvent *changeEvent = events->CreateEvent("vote_changed");
	changeEvent->SetInt("yesVotes", yesVotes);
	changeEvent->SetInt("noVotes", noVotes);
	changeEvent->SetInt("potentialVotes", totalClients);
	events->FireEvent(changeEvent);
}

unsigned int L4D2Vote::GetApproxMemUsage()
{
	return sizeof(L4D2Vote);
}

void L4D2Vote::DisplayVoteFail(BuiltinVoteFailReason reason)
{
	// reason is ignored in L4D2
	cell_t clients[256];
	unsigned int playersNum = GetAllPlayers(clients);

	int msgId = usermsgs->GetMessageIndex("VoteFail");

	bf_write *bf = usermsgs->StartMessage(msgId, clients, playersNum, USERMSG_RELIABLE);
	bf->WriteByte(TEAMS_ALL);
	usermsgs->EndMessage();
}

void L4D2Vote::DisplayVotePass(const char *winner)
{
	char *translation;
	switch(m_voteType)
	{
	case BuiltinVoteType_Kick:
		translation = TRANSLATION_L4D_VOTE_KICK_PASSED;
		break;

	case BuiltinVoteType_Restart:
		translation = TRANSLATION_L4D_VOTE_RESTART_PASSED;
		break;

	case BuiltinVoteType_ChangeLevel:
		translation = TRANSLATION_L4D_VOTE_CHANGELEVEL_PASSED;
		break;

	case BuiltinVoteType_ChangeCampaign:
		translation = TRANSLATION_L4D_VOTE_CHANGECAMPAIGN_PASSED;
		break;

	case BuiltinVoteType_ReturnToLobby:
		translation = TRANSLATION_L4D_VOTE_RETURNTOLOBBY_PASSED;
		break;

	case BuiltinVoteType_ChangeDifficulty:
		translation = TRANSLATION_L4D_VOTE_CHANGEDIFFICULTY_PASSED;
		break;

	case BuiltinVoteType_Custom:
		translation = TRANSLATION_L4D_VOTE_CUSTOM;
		break;

	}

	// Display the vote pass message
	cell_t clients[256];
	unsigned int playersNum = GetAllPlayers(clients);

	int msgId = usermsgs->GetMessageIndex("VotePass");

	bf_write *bf = usermsgs->StartMessage(msgId, clients, playersNum, USERMSG_RELIABLE);
	bf->WriteByte(TEAMS_ALL);
	bf->WriteString(translation);
	bf->WriteString(winner);
	usermsgs->EndMessage();
}

/*
bool L4D2Vote::SendOptions()
{
	// noop
	return false;
}
*/

L4D2Vote::L4D2Vote(IPluginFunction *pHandler, IdentityToken_t *pOwner, BuiltinVoteType voteType) :
BaseVote(pHandler, pOwner, voteType, true)
{
}

bool L4D2Vote::SetVoteType(BuiltinVoteType voteType)
{
	switch(voteType)
	{
	case BuiltinVoteType_Kick:
	case BuiltinVoteType_Restart:
	case BuiltinVoteType_ChangeLevel:
	case BuiltinVoteType_ChangeCampaign:
	case BuiltinVoteType_ReturnToLobby:
	case BuiltinVoteType_ChangeDifficulty:
	case BuiltinVoteType_Custom:
		m_voteType = voteType;
		return true;
		break;

	default:
		return false;
		break;
		
	}
}

void L4D2Vote::ShowVote(cell_t clients[], unsigned int playersNum, const char *voteArgument)
{
	char *translation;
	switch(m_voteType)
	{
	case BuiltinVoteType_Kick:
		translation = TRANSLATION_L4D_VOTE_KICK;
		break;

	case BuiltinVoteType_Restart:
		translation = TRANSLATION_L4D_VOTE_RESTART_START;
		break;

	case BuiltinVoteType_ChangeLevel:
		translation = TRANSLATION_L4D_VOTE_CHANGELEVEL_START;
		break;

	case BuiltinVoteType_ChangeCampaign:
		translation = TRANSLATION_L4D_VOTE_CHANGECAMPAIGN_START;
		break;

	case BuiltinVoteType_ReturnToLobby:
		translation = TRANSLATION_L4D_VOTE_RETURNTOLOBBY_START;
		break;

	case BuiltinVoteType_ChangeDifficulty:
		translation = TRANSLATION_L4D_VOTE_CHANGEDIFFICULTY_START;
		break;

	case BuiltinVoteType_Custom:
		translation = TRANSLATION_L4D_VOTE_CUSTOM;
		break;

	}

	int msgId = usermsgs->GetMessageIndex("VoteStart");

	bf_write *bf = usermsgs->StartMessage(msgId, clients, playersNum, USERMSG_RELIABLE);
	bf->WriteByte(TEAMS_ALL);
	bf->WriteByte(m_voteInitiator);
	bf->WriteString(translation);
	bf->WriteString(voteArgument);
	if (m_voteInitiator != SERVER_CLIENT_INDEX)
	{
		IGamePlayer *player = playerhelpers->GetGamePlayer(m_voteInitiator);
		if (player != NULL)
		{
			const char *playerName = player->GetName();
			bf->WriteString(playerName);
		}
		else
		{
			bf->WriteString("");
		}
	}
	else
	{
		// Don't write anything for the server
		bf->WriteString("");
	}
	usermsgs->EndMessage();
}

void L4D2Vote::ShowVoteCast(int entityId, int item)
{
	int msgId = usermsgs->GetMessageIndex("VoteRegistered");

	int vote = 0;

	switch (item)
	{
		case VOTE_YES_INDEX :
			vote = VOTE_YES; // Listed first (index 0), but value is 1
			break;

		case VOTE_NO_INDEX :
			vote = VOTE_NO; // Listed second (index 1), but value is 0
			break;

	}
	const cell_t players[] = { entityId };

	bf_write *bf = usermsgs->StartMessage(msgId, players, 1, USERMSG_RELIABLE);
	bf->WriteByte(vote);
	usermsgs->EndMessage();

}

void L4D2Vote::CastVote(edict_t *pEntity, const CCommand &args)
{
	if (!pEntity || pEntity->IsFree())
	{
		return;
	}

	const char *cmd = args.Arg(0);
	if (stricmp(cmd, "Vote") == 0)
	{

		// The text "Yes" or "No"
		int item;

		if (stricmp(args.Arg(1), "Yes"))
		{
			item = VOTE_YES_INDEX;
		}
		else
		{
			item = VOTE_NO_INDEX;
		}

		IGamePlayer *player = playerhelpers->GetGamePlayer(pEntity);
		
		int client = playerhelpers->GetClientOfUserId(player->GetUserId());

		ShowVoteCast(client, item);
		OnBuiltinVoteSelect(client, item);

		RETURN_META(MRES_SUPERCEDE);
	}

	RETURN_META(MRES_IGNORED);
}

void L4D2Vote::DisplayAllVotes(unsigned int totalClients, unsigned int yesVotes, unsigned int noVotes)
{
	IGameEvent *changeEvent = events->CreateEvent("vote_changed");
	changeEvent->SetInt("yesVotes", yesVotes);
	changeEvent->SetInt("noVotes", noVotes);
	changeEvent->SetInt("potentialVotes", totalClients);
	events->FireEvent(changeEvent);
}
