/**
 * vim: set ts=4 :
 * =============================================================================
 * Builtin Votes Extension
 * Copyright (C) 2011 Ross Bemrose (Powerlord).  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#include "extension.h"

/**
 * @file extension.cpp
 * @brief Implement extension code here.
 */
unsigned int m_Clients;
unsigned int m_TotalClients;
unsigned int m_Items;
bool m_bVoteStarted = false;
bool m_bCancelled = false;
CVector<unsigned int> m_Votes;
BaseVote *m_pCurVote;
unsigned int m_NumVotes;
unsigned int m_VoteTime;
float m_fStartTime;
int m_ClientVotes[256+1];
bool m_Revoting[256+1];
char m_leaderList[1024];
ITimer *m_displayTimer;

HandleType_t g_BuiltinVoteType;

#if SMINTERFACE_GAMEHELPERS_VERSION < 7
	int m_HinTextMsg;
	IGameConfig *g_pGameConf;
#endif

IPhraseCollection *g_pCorePhrases;
CGlobalVars *gpGlobals;
IServerGameClients *servergameclients;

ConVar *sm_vote_hintbox;
ConVar *sm_vote_chat;
ConVar *sm_vote_console;
ConVar *sm_vote_client_console;
ConVar *sm_vote_delay;
IGameEventManager2 *events;
ICvar *icvar;

// This will error out if you try to compile for lower source engines.  Since this ext won't work on them, I don't see this as a problem.
SH_DECL_HOOK2_void(IServerGameClients, ClientCommand, SH_NOATTRIB, 0, edict_t *, const CCommand &);

float g_next_vote = 0.0f;

BuiltinVotes g_BuiltinVotes;		/**< Global singleton for extension's main interface */

SMEXT_LINK(&g_BuiltinVotes);

void OnVoteDelayChange(IConVar *cvar, const char *value, float flOldValue)
{
	
	/* See if the new vote delay isn't something we need to account for */
	if (sm_vote_delay->GetFloat() < 1.0f)
	{
		g_next_vote = 0.0f;
		return;
	}

	/* If there was never a last vote, ignore this change */
	if (g_next_vote < 0.1f)
	{
		return;
	}

	/* Subtract the original value, then add the new one. */
	g_next_vote -= (float)atof(value);
	g_next_vote += sm_vote_delay->GetFloat();
}

int SortVoteItems(const void *item1, const void *item2)
{
	return ((builtinvote_result_t::item_vote_t *)item2)->count
		- ((builtinvote_result_t::item_vote_t *)item1)->count;
}

bool BuiltinVotes::SDK_OnMetamodLoad(SourceMM::ISmmAPI *ismm, char *error, size_t maxlen, bool late)
{
	// We need this interface for natives
	GET_V_IFACE_ANY(GetServerFactory, servergameclients, IServerGameClients, INTERFACEVERSION_SERVERGAMECLIENTS);
	GET_V_IFACE_CURRENT(GetEngineFactory, events, IGameEventManager2, INTERFACEVERSION_GAMEEVENTSMANAGER2);
	GET_V_IFACE_CURRENT(GetEngineFactory, icvar, ICvar, CVAR_INTERFACE_VERSION);

	// We need this object for certain handy variables
	gpGlobals = ismm->GetCGlobals();

	sm_vote_delay->InstallChangeCallback(OnVoteDelayChange);
	return true;
}

bool BuiltinVotes::SDK_OnLoad(char *error, size_t maxlen, bool late)
{
#if SMINTERFACE_GAMEHELPERS_VERSION < 7
	m_HinTextMsg = usermsgs->GetMessageIndex("HintText");

	// We need this solely for the hint messages we send
	gameconfs->LoadGameConfigFile("core.games", &g_pGameConf, error, maxlen);
#endif

	HandleAccess access;
	handlesys->InitAccessDefaults(NULL, &access);

	/* Deny cloning to menus */
	access.access[HandleAccess_Clone] = HANDLE_RESTRICT_OWNER|HANDLE_RESTRICT_IDENTITY;
	
	g_BuiltinVoteType = handlesys->CreateType("BuiltinVoteType", this, 0, NULL, &access, myself->GetIdentity(), NULL);

	g_pCorePhrases = translator->CreatePhraseCollection();
	g_pCorePhrases->AddPhraseFile("core.phrases");

	playerhelpers->AddClientListener(this);
	return true;
}

bool BuiltinVotes::CoreTranslate(char *buffer, 
				   size_t maxlength,
				   const char *format,
				   unsigned int numparams,
				   size_t *pOutLength,
				   ...)
{
	va_list ap;
	unsigned int i;
	const char *fail_phrase;
	void *params[MAX_TRANSLATE_PARAMS];

	if (numparams > MAX_TRANSLATE_PARAMS)
	{
		assert(false);
		return false;
	}

	va_start(ap, pOutLength);
	for (i = 0; i < numparams; i++)
	{
		params[i] = va_arg(ap, void *);
	}
	va_end(ap);

	if (!g_pCorePhrases->FormatString(buffer,
		maxlength, 
		format, 
		params,
		numparams,
		pOutLength,
		&fail_phrase))
	{
		if (fail_phrase != NULL)
		{
			Msg("[SM] Could not find core phrase: %s", fail_phrase);
		}
		else
		{
			Msg("[SM] Unknown fatal error while translating a core phrase.");
		}

		return false;
	}

	return true;
}

#if SMINTERFACE_GAMEHELPERS_VERSION < 7
// This was added to the IGameHelpers interface in version 7
// Modified from HintTextMsg in HalfLife2.h
bool BuiltinVotes::HintTextMsg(int client, const char *msg)
{
	bf_write *pBitBuf = NULL;
	cell_t players[] = {client};

	if ((pBitBuf =  usermsgs->StartMessage(m_HinTextMsg, players, 1, USERMSG_RELIABLE)) == NULL)
	{
		return false;
	}

	const char *pre_byte = g_pGameConf->GetKeyValue("HintTextPreByte");
	if (pre_byte != NULL && strcmp(pre_byte, "yes") == 0)
	{
		pBitBuf->WriteByte(1);
	}
	pBitBuf->WriteString(msg);
	usermsgs->EndMessage();

	return true;
}
#endif

void BuiltinVotes::DrawHintProgress()
{
	if (!sm_vote_hintbox->GetBool())
	{
		return;
	}

	static char buffer[1024];

	float timeRemaining = (m_fStartTime + m_VoteTime) - gpGlobals->curtime;
	if (timeRemaining < 0)
	{
		timeRemaining = 0.0;
	}

	int iTimeRemaining = RoundFloatToInt(timeRemaining);

	int maxclients = gpGlobals->maxClients;
	for (int i = 1; i <= maxclients; i++)
	{
		if (playerhelpers->GetGamePlayer(i)->IsInGame())
		{
			CoreTranslate(buffer, sizeof(buffer), "%T%s", 6, NULL, "Vote Count", &i, &m_NumVotes, &m_TotalClients, &iTimeRemaining, &m_leaderList);
#if SMINTERFACE_GAMEHELPERS_VERSION < 7
			HintTextMsg(i, buffer);
#else
			gamehelpers->HintTextMsg(i, buffer);
#endif
		}
	}
}

void BuiltinVotes::InternalReset()
{
	m_Clients = 0;
	m_Items = 0;
	m_bVoteStarted = false;
	m_pCurVote = NULL;
	m_NumVotes = 0;
	m_bCancelled = false;
	m_leaderList[0] = '\0';
	m_displayTimer = NULL;
	m_TotalClients = 0;
}

bool BuiltinVotes::StartVote(BaseVote *vote, unsigned int num_clients, int clients[], unsigned int max_time)
{
	if (!InitializeVoting(vote, max_time))
	{
		return false;
	}

	/* Note: we can use game time and not universal time because
	 * if we're voting then players are in-game
	 */

	float fVoteDelay = sm_vote_delay->GetFloat();
	if (fVoteDelay < 1.0)
	{
		g_next_vote = 0.0;
	}
	else
	{
		/* This little trick breaks for infinite votes!
		 * However, we just ignore that since those 1) shouldn't exist and
		 * 2) people must be checking IsVoteInProgress() beforehand anyway.
		 */
		g_next_vote = gpGlobals->curtime + fVoteDelay + (float)max_time;
	}

	m_fStartTime = gpGlobals->curtime;
	m_VoteTime = max_time;
	StartVoting();

	vote->SendOptions(); // No-op for L4D/L4D2, but we need it for TF2
	vote->ShowVote(clients, num_clients);

	return true;
}

bool BuiltinVotes::IsVoteInProgress()
{
	return m_bVoteStarted;
}

bool BuiltinVotes::InitializeVoting(BaseVote *vote, unsigned int time)
{
	if (IsVoteInProgress())
	{
		return false;
	}

	InternalReset();

	for (int i = 1; i <= gpGlobals->maxClients; i++)
	{
		m_ClientVotes[i] = VOTE_NOT_VOTING;
		m_Revoting[i] = false;
	}

	m_Items = vote->GetItemCount();

	if (m_Votes.size() < (size_t)m_Items)
	{
		/* Only clear the items we need to... */
		size_t size = m_Votes.size();
		for (size_t i = 0; i < size; i++)
		{
			m_Votes[i] = 0;
		}
		m_Votes.resize(m_Items, 0);
	}
	else
	{
		for (unsigned int i = 0; i < m_Items; i++)
		{
			m_Votes[i] = 0;
		}
	}
	
	m_pCurVote = vote;
	m_VoteTime = time;

	return true;
}

void BuiltinVotes::EndVoting()
{
	float fVoteDelay = sm_vote_delay->GetFloat();
	if (fVoteDelay < 1.0)
	{
		g_next_vote = 0.0;
	}
	else
	{
		g_next_vote = gpGlobals->curtime + fVoteDelay;
	}

	if (m_displayTimer)
	{
		SH_REMOVE_HOOK(IServerGameClients, ClientCommand, servergameclients, SH_MEMBER(m_pCurVote, &BaseVote::CastVote), false);
		timersys->KillTimer(m_displayTimer);
	}

	if (m_bCancelled)
	{
		/* If we were cancelled, don't bother tabulating anything.
		 * Reset just in case someone tried to redraw, which means
		 * we need to save out states.
		 */
		BaseVote *vote = m_pCurVote;
		InternalReset();

		vote->OnBuiltinVoteCancel(BuiltinVoteFail_Generic);
		vote->OnBuiltinVoteEnd(BuiltinVoteEnd_Cancelled);
		return;
	}

	// Yes, another variable named vote
	builtinvote_result_t vote;
	builtinvote_result_t::client_vote_t client_vote[256];
	builtinvote_result_t::item_vote_t item_vote[256];

	memset(&vote, 0, sizeof(vote));

	/* Build the item list */
	for (unsigned int i=0; i<m_Items; i++)
	{
		if (m_Votes[i] > 0)
		{
			item_vote[vote.num_items].count = m_Votes[i];
			item_vote[vote.num_items].item = i;
			vote.num_votes += m_Votes[i];
			vote.num_items++;
		}
	}
	vote.item_list = item_vote;

	if (!vote.num_votes)
	{
		BaseVote *menu = m_pCurVote;
		InternalReset();
		menu->OnBuiltinVoteCancel(BuiltinVoteFail_NotEnoughVotes);
		menu->OnBuiltinVoteEnd(BuiltinVoteEnd_Cancelled);
		return;
	}

	/* Build the client list */
	for (int i=1; i<=gpGlobals->maxClients; i++)
	{
		if (m_ClientVotes[i] >= VOTE_PENDING)
		{
			client_vote[vote.num_clients].client = i;
			client_vote[vote.num_clients].item = m_ClientVotes[i];
			vote.num_clients++;
		}
	}
	vote.client_list = client_vote;

	/* Sort the item list descending like we promised */
	qsort(item_vote,
		vote.num_items,
		sizeof(builtinvote_result_t::item_vote_t),
		SortVoteItems);

	/* Save states, then clear what we've saved.
	 * This make us re-entrant, which is always the safe way to go.
	 */
	BaseVote *menu = m_pCurVote;
	InternalReset();

	/* Send vote info */
	menu->OnBuiltinVoteResults(&vote);
	menu->OnBuiltinVoteEnd(BuiltinVoteEnd_Done);
}

void BuiltinVotes::StartVoting()
{
	if(!m_pCurVote)
	{
		return;
	}

	m_bVoteStarted = true;

	m_displayTimer = timersys->CreateTimer(this, 1.0, NULL, TIMER_FLAG_REPEAT|TIMER_FLAG_NO_MAPCHANGE);

	/* By now we know how many clients were set.
	 * If there are none, we should end IMMEDIATELY.
	 */
	if (m_Clients == 0)
	{
		EndVoting();
	}

	m_TotalClients = m_Clients;

}

ResultType BuiltinVotes::OnTimer(ITimer *pTimer, void *pData)
{
	DrawHintProgress();

	return Pl_Continue;
}

void BuiltinVotes::OnTimerEnd(SourceMod::ITimer *pTimer, void *pData)
{
	m_displayTimer = NULL;
}

void BuiltinVotes::OnClientDisconnected(int client)
{
	if (!IsVoteInProgress())
	{
		return;
	}

	/* Wipe out their vote if they had one.  We have to make sure the the the
	 * newly connected client is not allowed to vote.
	 */
	int item;
	if ((item = m_ClientVotes[client]) >= VOTE_PENDING)
	{
		if (item >= 0)
		{
			assert((unsigned)item < m_Items);
			assert(m_Votes[item] > 0);
			m_Votes[item]--;
		}
		m_ClientVotes[client] = VOTE_NOT_VOTING;
	}

}

HandleError BuiltinVotes::ReadBuiltinVoteHandle(Handle_t handle, BaseVote **vote)
{
	HandleSecurity sec;

	sec.pIdentity = myself->GetIdentity();
	sec.pOwner = NULL;

	return handlesys->ReadHandle(handle, g_BuiltinVoteType, &sec, (void **)vote);
}

void OnHandleDestroy(HandleType_t type, void *object)
{
	BaseVote *vote = (BaseVote *)object;
	vote->Destroy(false);
}

bool GetHandleApproxSize(HandleType_t type, void *object, unsigned int *pSize)
{
	*pSize = ((BaseVote *)object)->GetApproxMemUsage();
	return true;
}



/***********************************
 **** NATIVE DEFINITIONS ***********
 ***********************************/

cell_t NativeCreateBuiltinVote(IPluginContext *pContext, const cell_t *params)
{
	IPluginFunction *pFunction;
	if ((pFunction = pContext->GetFunctionById(params[1])) == NULL)
	{
		return pContext->ThrowNativeError("Function id %x is invalid", params[1]);
	}

	BuiltinVoteType voteType = (BuiltinVoteType)params[2];

#if SOURCE_ENGINE == SE_ORANGEBOXVALVE
	bool bYesNo = params[3] != 0;
	BaseVote *vote = new TF2Vote(pFunction, pContext->GetIdentity(), voteType, bYesNo);
#elif SOURCE_ENGINE == SE_LEFT4DEAD
	BaseVote *vote = new L4DVote(pFunction, pContext->GetIdentity(), voteType);
#elif SOURCE_ENGINE == SE_LEFT4DEAD2
	BaseVote *vote = new L4D2Vote(pFunction, pContext->GetIdentity(), voteType);
#endif

	if (!vote)
		return BAD_HANDLE;

	Handle_t hndl = vote->GetHandle();
	if (!hndl)
	{
		vote->Destroy(false);
		return BAD_HANDLE;
	}

	return hndl;
}

cell_t NativeDisplayBuiltinVote(IPluginContext *pContext, const cell_t *params)
{
	if (IsVoteInProgress())
	{
		return pContext->ThrowNativeError("A vote is already in progress");
	}

	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	cell_t *addr;
	pContext->LocalToPhysAddr(params[2], &addr);

	if (!g_BuiltinVotes.StartVote(vote, params[3], addr, params[4]))
	{
		return 0;
	}

	return 1;
}

cell_t NativeAddBuiltinVoteItem(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	char *info;

	pContext->LocalToString(params[2], &info);
	return vote->AppendItem(info) ? 1 : 0;
}

cell_t NativeInsertBuiltinVoteItem(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Menu handle %x is invalid (error %d)", hndl, err);
	}

	char *info;

	pContext->LocalToString(params[3], &info);

	return vote->InsertItem(params[2], info) ? 1 : 0;
}

cell_t NativeRemoveBuiltinVoteItem(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	return vote->RemoveItem(params[2]) ? 1 : 0;
}

cell_t NativeRemoveAllBuiltinVoteItems(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	vote->RemoveAllItems();

	return 1;
}

cell_t NativeGetBuiltinVoteItem(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	const char *info;

	if ((info=vote->GetItemInfo(params[2])) == NULL)
	{
		return 0;
	}

	pContext->StringToLocalUTF8(params[3], params[4], info, NULL);

	return 1;
}

cell_t NativeGetBuiltinVoteItemCount(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	return vote->GetItemCount();
}

/*
cell_t NativeSetTranslation(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	char *buffer;
	pContext->LocalToString(params[2], &buffer);

	vote->SetTranslation(buffer);

	return 1;
}

cell_t NativeGetTranslation(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	size_t written;
	const char *translation = vote->GetTranslation();
	pContext->StringToLocalUTF8(params[2], params[3], translation, &written);

	return (cell_t)written;
}

cell_t NativeSetPassedTranslation(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	char *buffer;
	pContext->LocalToString(params[2], &buffer);

	vote->SetPassedTranslation(buffer);

	return 1;
}

cell_t NativeGetPassedTranslation(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	size_t written;
	const char *translation = vote->GetPassedTranslation();
	pContext->StringToLocalUTF8(params[2], params[3], translation, &written);

	return (cell_t)written;
}
*/

cell_t NativeSetVoteType(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	BuiltinVoteType voteType = (BuiltinVoteType)params[2];

	return 1;
}

cell_t NativeGetVoteType(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	return vote->GetVoteType();
}

cell_t NativeSetVoteArgument(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	char *buffer;
	pContext->LocalToString(params[2], &buffer);

	vote->SetVoteArgument(buffer);
	
	return 1;

}

cell_t NativeGetVoteArgument(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	size_t written;
	const char *argument = vote->GetVoteArgument();
	pContext->StringToLocalUTF8(params[2], params[3], argument, &written);

	return (cell_t)written;
}


/*
cell_t NativeCancelVote(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	g_BuiltinVotes.CancelVote(vote);

	return 1;
}
*/

cell_t NativeIsVoteInProgress(IPluginContext *pContext, const cell_t *params)
{
	return g_BuiltinVotes.IsVoteInProgress() ? 1 : 0;
}

// Not implemented yet

/*
cell_t NativeGetVoteOptionFlags(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	return vote->GetVoteOptionFlags();
}

cell_t NativeSetVoteOptionFlags(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	vote->SetVoteOptionFlags(params[2]);

	return 1;
}
*/

cell_t NativeCancelVoting(IPluginContext *pContext, const cell_t *params)
{

	if (!g_BuiltinVotes.IsVoteInProgress())
	{
		return pContext->ThrowNativeError("No vote is in progress");
	}

	g_BuiltinVotes.CancelVoting();

	return 1;
}

cell_t NativeSetVoteResultCallback(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	IPluginFunction *pFunction = pContext->GetFunctionById(params[2]);
	if (!pFunction)
	{
		return pContext->ThrowNativeError("Invalid function %x", params[2]);
	}

	vote->SetResultHandler(pFunction);

	/*
	void *array[2];
	array[0] = pFunction;
	array[1] = (void *) &params[2];

	IBuiltinVoteHandler *pHandler = vote->GetHandler();
	if (!pHandler->OnBuiltinVoteSetHandlerOption("set_vote_results_handler", (const void *)array))
	{
		return pContext->ThrowNativeError("The given vote does not support this option");
	}
	*/

	return 1;
}

cell_t NativeCheckVoteDelay(IPluginContext *pContext, const cell_t *params)
{
	return g_BuiltinVotes.GetRemainingVoteDelay();
}

cell_t NativeIsClientInVotePool(IPluginContext *pContext, const cell_t *params)
{
	int client;
	IGamePlayer *pPlayer;

	client = params[1];

	if ((pPlayer = playerhelpers->GetGamePlayer(client)) == NULL)
	{
		return pContext->ThrowNativeError("Invalid client index %d", client);
	}

	if (!g_BuiltinVotes.IsVoteInProgress())
	{
		return pContext->ThrowNativeError("No vote is in progress");
	}

	return g_BuiltinVotes.IsClientInVotePool(client) ? 1 : 0;
}

cell_t NativeRedrawClientVoteMenu(IPluginContext *pContext, const cell_t *params)
{
	int client;
	IGamePlayer *pPlayer;

	client = params[1];
	if ((pPlayer = playerhelpers->GetGamePlayer(client)) == NULL)
	{
		return pContext->ThrowNativeError("Invalid client index %d", client);
	}

	if (!g_BuiltinVotes.IsVoteInProgress())
	{
		return pContext->ThrowNativeError("No vote is in progress");
	}

	if (!g_BuiltinVotes.IsClientInVotePool(client))
	{
		return pContext->ThrowNativeError("Client is not in the voting pool");
	}

	bool revote = true;
	if (params[0] >= 2 && !params[2])
	{
		revote = false;
	}

	return g_BuiltinVotes.RedrawClientVote(client, revote) ? 1 : 0;
}

cell_t NativeDisplayVoteFail(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	BuiltinVoteFailReason reason = (BuiltinVoteFailReason) params[1];

	vote->DisplayVoteFail(reason);
	return 1;
}

cell_t NativeDisplayVotePass(IPluginContext *pContext, const cell_t *params)
{
	Handle_t hndl = (Handle_t)params[1];
	HandleError err;
	BaseVote *vote;

	if ((err=g_BuiltinVotes.ReadBuiltinVoteHandle(params[1], &vote)) != HandleError_None)
	{
		return pContext->ThrowNativeError("Vote handle %x is invalid (error %d)", hndl, err);
	}

	char *winner;
	pContext->LocalToString(params[2], &winner);

	vote->DisplayVotePass(winner);
	return 1;
}
