/**
 * vim: set ts=4 :
 * =============================================================================
 * Builtin Votes Extension
 * Copyright (C) 2011 Ross Bemrose (Powerlord).  All rights reserved.
 * =============================================================================
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 * Version: $Id$
 */

#ifndef _INCLUDE_SOURCEMOD_EXTENSION_PROPER_H_
#define _INCLUDE_SOURCEMOD_EXTENSION_PROPER_H_

/**
 * @file extension.h
 * @brief Sample extension code header.
 */

#include "smsdk_ext.h"
#include "Votes.h"
#include "igameevents.h"

#define VOTE_NOT_VOTING -2
#define VOTE_PENDING -1

#define MAX_TRANSLATE_PARAMS		32

/**
 * @brief Sample implementation of the SDK Extension.
 * Note: Uncomment one of the pre-defined virtual functions in order to use it.
 */
class BuiltinVotes : public SDKExtension,
	public ITimedEvent,
	public IClientListener,
	public IHandleTypeDispatch

{
public:
	/**
	 * @brief This is called after the initial loading sequence has been processed.
	 *
	 * @param error		Error message buffer.
	 * @param maxlength	Size of error message buffer.
	 * @param late		Whether or not the module was loaded after map load.
	 * @return			True to succeed loading, false to fail.
	 */
	virtual bool SDK_OnLoad(char *error, size_t maxlength, bool late);
	
	/**
	 * @brief This is called right before the extension is unloaded.
	 */
	//virtual void SDK_OnUnload();

	/**
	 * @brief This is called once all known extensions have been loaded.
	 * Note: It is is a good idea to add natives here, if any are provided.
	 */
	//virtual void SDK_OnAllLoaded();

	/**
	 * @brief Called when the pause state is changed.
	 */
	//virtual void SDK_OnPauseChange(bool paused);

	/**
	 * @brief this is called when Core wants to know if your extension is working.
	 *
	 * @param error		Error message buffer.
	 * @param maxlength	Size of error message buffer.
	 * @return			True if working, false otherwise.
	 */
	//virtual bool QueryRunning(char *error, size_t maxlength);
public:
#if defined SMEXT_CONF_METAMOD
	/**
	 * @brief Called when Metamod is attached, before the extension version is called.
	 *
	 * @param error			Error buffer.
	 * @param maxlength		Maximum size of error buffer.
	 * @param late			Whether or not Metamod considers this a late load.
	 * @return				True to succeed, false to fail.
	 */
	virtual bool SDK_OnMetamodLoad(ISmmAPI *ismm, char *error, size_t maxlen, bool late);

	/**
	 * @brief Called when Metamod is detaching, after the extension version is called.
	 * NOTE: By default this is blocked unless sent from SourceMod.
	 *
	 * @param error			Error buffer.
	 * @param maxlength		Maximum size of error buffer.
	 * @return				True to succeed, false to fail.
	 */
	//virtual bool SDK_OnMetamodUnload(char *error, size_t maxlength);

	/**
	 * @brief Called when Metamod's pause state is changing.
	 * NOTE: By default this is blocked unless sent from SourceMod.
	 *
	 * @param paused		Pause state being set.
	 * @param error			Error buffer.
	 * @param maxlength		Maximum size of error buffer.
	 * @return				True to succeed, false to fail.
	 */
	//virtual bool SDK_OnMetamodPauseChange(bool paused, char *error, size_t maxlength);
#endif
public: // ITimedEvent
	ResultType OnTimer(ITimer *pTimer, void *pData);
	void OnTimerEnd(ITimer *pTimer, void *pData);
public: // IClientListener
	void OnClientDisconnected(int client);
public: // IHandleTypeDispatch
	void OnHandleDestroy(HandleType_t type, void *object);
	bool GetHandleApproxSize(HandleType_t type, void *object, unsigned int *pSize);
public:
	#if SMINTERFACE_GAMEHELPERS_VERSION < 7
// In SourceMod 1.3.x this doesn't exist in IGameHelpers
	bool HintTextMsg(int client, const char *msg);
#endif
	void InternalReset();
	bool CoreTranslate(char *buffer, 
				   size_t maxlength,
				   const char *format,
				   unsigned int numparams,
				   size_t *pOutLength,
				   ...);
	void DrawHintProgress();
	void EndVoting();
	bool StartVote(BaseVote *vote, unsigned int num_clients, int clients[], unsigned int max_time);
	bool IsVoteInProgress();
	void StartVoting();
	HandleError ReadBuiltinVoteHandle(Handle_t handle, BaseVote **vote);
private:
	bool InitializeVoting(BaseVote *vote, unsigned int time);
};

// Natives.  They always have the same format.
cell_t NativeCreateBuiltinVote(IPluginContext *pContext, const cell_t *params);
cell_t NativeDisplayBuiltinVote(IPluginContext *pContext, const cell_t *params);
cell_t NativeAddBuiltinVoteItem(IPluginContext *pContext, const cell_t *params);
cell_t NativeInsertBuiltinVoteItem(IPluginContext *pContext, const cell_t *params);
cell_t NativeRemoveBuiltinVoteItem(IPluginContext *pContext, const cell_t *params);
cell_t NativeRemoveAllBuiltinVoteItems(IPluginContext *pContext, const cell_t *params);
cell_t NativeGetBuiltinVoteItem(IPluginContext *pContext, const cell_t *params);
cell_t NativeGetBuiltinVoteItemCount(IPluginContext *pContext, const cell_t *params);
cell_t NativeSetVoteType(IPluginContext *pContext, const cell_t *params);
cell_t NativeGetVoteType(IPluginContext *pContext, const cell_t *params);
cell_t NativeSetVoteArgument(IPluginContext *pContext, const cell_t *params);
cell_t NativeGetVoteArgument(IPluginContext *pContext, const cell_t *params);
//cell_t NativeSetTranslation(IPluginContext *pContext, const cell_t *params);
//cell_t NativeGetTranslation(IPluginContext *pContext, const cell_t *params);
//cell_t NativeSetPassedTranslation(IPluginContext *pContext, const cell_t *params);
//cell_t NativeGetPassedTranslation(IPluginContext *pContext, const cell_t *params);
//cell_t NativeCancelVote(IPluginContext *pContext, const cell_t *params);
cell_t NativeIsVoteInProgress(IPluginContext *pContext, const cell_t *params);
//cell_t NativeGetVoteOptionFlags(IPluginContext *pContext, const cell_t *params);
//cell_t NativeSetVoteOptionFlags(IPluginContext *pContext, const cell_t *params);
cell_t NativeCancelVoting(IPluginContext *pContext, const cell_t *params);
cell_t NativeSetVoteResultCallback(IPluginContext *pContext, const cell_t *params);
cell_t NativeCheckVoteDelay(IPluginContext *pContext, const cell_t *params);
cell_t NativeIsClientInVotePool(IPluginContext *pContext, const cell_t *params);
cell_t NativeRedrawClientVoteMenu(IPluginContext *pContext, const cell_t *params);
cell_t NativeDisplayVoteFail(IPluginContext *pContext, const cell_t *params);
cell_t NativeDisplayVotePass(IPluginContext *pContext, const cell_t *params);

const sp_nativeinfo_t BuiltinVoteNatives[] =
{
	{"CreateBuiltinVote",				NativeCreateBuiltinVote},
	{"DisplayBuiltinVoteFail",			NativeDisplayVoteFail},
	{"DisplayBuiltinVotePass",			NativeDisplayVotePass},
	{"AddBuiltinVoteItem",				NativeAddBuiltinVoteItem},
	{"InsertBuiltinVoteItem",			NativeInsertBuiltinVoteItem},
	{"RemoveBuiltinVoteItem",			NativeRemoveBuiltinVoteItem},
	{"RemoveAllBuiltinVoteItems",		NativeRemoveAllBuiltinVoteItems},
	{"GetBuiltinVoteItem",				NativeGetBuiltinVoteItem},
	{"GetBuiltinVoteItemCount",			NativeGetBuiltinVoteItemCount},
	{"IsBuiltinVoteInProgress",			NativeIsVoteInProgress},
	{"CancelBuiltinVote",				NativeCancelVoting},
	{"DisplayBuiltinVote",				NativeDisplayBuiltinVote},
	{"SetBuiltinVoteResultCallback",	NativeSetVoteResultCallback},
	{"CheckBuiltinVoteDelay",			NativeCheckVoteDelay},
	{"IsClientInBuiltinVotePool",		NativeIsClientInVotePool},
	{"RedrawClientBuiltinVoteMenu",		NativeRedrawClientVoteMenu},
	{NULL,								NULL},
};

#endif // _INCLUDE_SOURCEMOD_EXTENSION_PROPER_H_
